/***********************************************************************
**
** crt0.ld:  CoolRisc 816 linker script
**
**
** Modifications:
**
** 20-05-1998/Maup: First definition for EM6812
***********************************************************************/

/***********************************************************************
**
** Linker script functions
**
** Initialize the different memory areas
** Give to each peripherical register an adress
** Relocate stack pointer
** Place each compiler section into a certain memory area 
**
** Memory area definition 
** 
** prog_rom : program memory space (8k max = 0x7fff)
** ram_lp	: internal data low-power RAM page 0 space
** ram_dp	: internal data dual port RAM page 0 space 
** periph_1	: internal peripheral addresses page 0 space
** dead zone 
** periph_2	: internal peripheral addresses page 0 space
** dead zone
** periph_3	: internal peripheral addresses page 0 space
** dead zone
** periph_4	: internal peripheral addresses page 0 space
** ram_1	: internal data RAM page 0 space
** ram_2	: internal data RAM space
**
** Sections definition: 
** 
** .text         : program space, ELF's 0x100000..0x107fff addressing range
** .rodata       : read-only data
** .data         : data with initialization value
** .bss          : data without initialization value
** .page0_data   : page 0 data with initialization value
** .page0_bss    : page 0 data without initialization value
**
***********************************************************************/

MEMORY
{
  prog_rom      	  : ORIGIN = 0x00100000, LENGTH = 4*8K   
  ram_lp		        : ORIGIN = 0x00000000, LENGTH = 12
  ram_dp		        : ORIGIN = 0x0000000C, LENGTH = 4
  periph_1		      : ORIGIN = 0x00000010, LENGTH = 14	       
  periph_2		      : ORIGIN = 0x00000020, LENGTH = 12
  periph_3		      : ORIGIN = 0x00000030, LENGTH = 13
  periph_4		      : ORIGIN = 0x0000004E, LENGTH = 18
  ram_1			        : ORIGIN = 0x00000060, LENGTH = 0xa0 /* 160 */
  ram_2			        : ORIGIN = 0x00000100, LENGTH = 0x100 /* 96 */
  error_nodata_rom	: ORIGIN = 0x00001000, LENGTH = 0
 }
 
 
/***********************************************************************
**
** Peripherical register address definition
**
***********************************************************************/

 /* 
    In this default ctrl0.ld file, ram_lp and ram_dp are let undefined. They
    can be defined as registers or ram section. It is let to the user's 
    choice. 
 */ 

/************************************/
/* new names */
/************************************/

RegRam0   = 0x0C;
RegRam1   = 0x0D;
RegRam2   = 0x0E;
RegRam3   = 0x0F;
 
/*** System registers *****/

RegSys1           = 0x10; 
RegSys2           = 0x11;
RegResStat        = 0x12;
RegTrimRC         = 0x13;
RegPrscCkSel      = 0x14;
RegPr1Status      = 0x15;
RegPr2Status      = 0x16;
RegInt20          = 0x17;
RegInt10          = 0x18;
RegInt00          = 0x19;
RegMsk20          = 0x1A;
RegMsk10          = 0x1B;
RegMsk00          = 0x1C;

/********* Analog *********/

RegAnaCfg         = 0x20;

/********* Port A *********/

RegInPA           = 0x21;
RegOutPA          = 0x22;
RegCfgPA          = 0x23;
RegIOSelPA        = 0x24;
RegCmbKey         = 0x25;
RegMskRstWkUp     = 0x26;
RegIntEdgPAPA     = 0x27;
RegEnDebPA        = 0x28;
RegOpenDrainPA    = 0x29;
RegPullUpPA       = 0x2A;
RegPullDownPA     = 0x2B;
    
/********* Port B *********/

RegInPB           = 0x30;
RegOutPB          = 0x31;
RegCfgPB          = 0x32;
RegSigSel         = 0x33;
RegIOSelPB        = 0x34;
RegOpenDrainPB    = 0x35;
RegPullUpPB       = 0x36;
RegPullDownPB     = 0x37;

/********* SPI *********/
      
RegSPICfg         = 0x38;
RegSPIDat         = 0x39;
RegSPILoad        = 0x3A;

/********* WD *********/

RegWDSys          = 0x3B;
RegWDKey          = 0x3C;

/***** General Timer ****/

RegTimersCfg      = 0x4E;
RegTimersStart    = 0x4F;

/********* Timer1 *********/

RegTim1Cfg        = 0x50;
RegTim1Status     = 0x51;
RegTim1Load       = 0x52;
RegTim1Comp       = 0x53;

/********* Timer2 *********/

RegTim2Cfg        = 0x54;
RegTim2Status     = 0x55;
RegTim2Load       = 0x56;
RegTim2Comp       = 0x57;

/********* Timer3 *********/

RegTim3Cfg        = 0x58;
RegTim3Status     = 0x59;
RegTim3Load       = 0x5A;
RegTim3Comp       = 0x5B;

/********* Timer4 *********/

RegTim4Cfg        = 0x5C;
RegTim4Status     = 0x5D;
RegTim4Load       = 0x5E;
RegTim4Comp       = 0x5F;





/************************************/
/* old names */
/************************************/

 
/********* System registers *********/

RegSys1	   	  = 	0x10;	
RegSys2  	    =	0x11;
RegResStat  	=	0x12;
RegTrimRC   	=	0x13;
RegPrscCkSel   	=	0x14;
RegPresc1Status	=	0x15;
RegPresc2Status	=	0x16;
RegInt20   	=	0x17;
RegInt10   	=	0x18;
RegInt00   	=	0x19;
RegMsk20   	=	0x1A;
RegMsk10   	=	0x1B;
RegMsk00   	=	0x1C;
RegGB		=	0x1D;	

/********* Analog *********/

RegAnaCfg      	=	0x20;

/********* Port A *********/

RegInPA		=	0x21;
RegOutPA   	=	0x22;
RegCfgPA 	=	0x23;
RegIOSelPA    	=	0x24;
RegCmbPA    	=	0x25;
RegMskPA	=	0x26;
RegIntEdgPA   	=	0x27;
RegEnDebPA  	=	0x28;
RegOpenDrainPA	=	0x29;
RegPullUpPA   	=	0x2A;
RegPullDownPA	=	0x2B;
		
/********* Port B *********/

RegInPB   	=	0x30;
RegOutPB  	=	0x31;
RegCfgPB  	=	0x32;
RegIntSel 	=	0x33;
RegIOSelPB	=	0x34;
RegOpenDrainPB	=	0x35;
RegPullUpPB   	=	0x36;
RegPullDownPB 	=	0x37;

/********* SPI *********/
			
RegSPICfg 	=	0x38;
RegSPIDat 	=	0x39;
RegSPILoad	=	0x3A;

/********* WD *********/

RegWDSys  	=	0x3B;
RegWDKey  	=	0x3C;

/********* General Timer *********/

RegTimersCfg	=      	0x4E;
RegTimersStart	=      	0x4F;

/********* Timer1 *********/

RegTim1Cfg      =	0x50;
RegTim1Status   =	0x51;
RegTim1Load     =	0x52;
RegTim1Comp     =	0x53;

/********* Timer2 *********/

RegTim2Cfg 	=	0x54;
RegTim2Status	=	0x55;
RegTim2Load	=	0x56;
RegTim2Comp     =	0x57;

/********* Timer3 *********/

RegTim3Cfg 	=	0x58;
RegTim3Status	=	0x59;
RegTim3Load	=	0x5A;
RegTim3Comp     =	0x5B;

/********* Timer4 *********/

RegTim4Cfg 	=	0x5C;
RegTim4Status	=	0x5D;
RegTim4Load	=	0x5E;
RegTim4Comp     =	0x5F;

/***********************************************************************
** Definition of the top of the stack
***********************************************************************/
/* stack is on RAM_2 */
_estack = 0x01FF ;   

/***********************************************************************
**
** Place compiler sections into the various memory area
**
***********************************************************************/ 
SECTIONS 
{
    /* THE PROGRAM CODE IS STORED IN THE .TEXT SECTION */
    .text : 
    {
        _stext = .;
        *(.text )
        _etext = .;
        _eprom = .;
    } > prog_rom
    
    /* THE PAGE 0 UNINITIALIZED DATA ARE STORED INTO THE .PAGE0_BSS SECTION */
    .page0_bss :  
    {
        _spage0bss = .;
        *(.page0_bss)
        _epage0bss = .;
    } > ram_1

    /* THE UNINITIALIZED DATA ARE STORED INTO THE .BSS SECTION */
    .bss : 
    {
        _sbss = .;
        *(.bss)
        *(COMMON)
        _ebss = .;
    } > ram_2
    
    /* UNUSED SECTIONS, JUST FOR INFORMATION. SHOULD NOT APPEAR => MISTAKE */
    
    .rodata : /*rutt AT ( ADDR(.bss)+ SIZEOF(.bss)) */
    {
        _srodata = .;
        *(.rodata)
        _erodata = .;
    } > error_nodata_rom

    .page0_data  :  /*rutt AT (ADDR (.rodata) + SIZEOF(.rodata)) */
    {
		_spage0data = .;
		*(.page0_data)
		_epage0data = .;
	  } > error_nodata_rom
    
    .data  :  AT (ADDR (.rodata) + SIZEOF(.rodata)+ SIZEOF(.page0_data))
    {
		_sdata = .;
		*(.data)
		_edata = .;
	  } > error_nodata_rom

    /* THESE SECTIONS CONTAIN DEBUG INFORMATION */
    
    .stab : 
    {
        *(.stab)
    }   

    .stabstr : 
    {
        *(.stabstr)
    }       
}


