;***********************************************************************
;
; crt0.s:   startup module
;
; Modifications:
;
; 20-05-2003/Maup: Initialization code
;
;**********************************************************************
	.include "definition.s"
   
   .altregsyn	0	
	.global 	_start
	.global 	main
  .text
  .org  0

  
;**********************************************************************
; Reset & interrupt vectors
;**********************************************************************
_start:

 	jump    main_init	; reset
	jump	__int1_h	; interrupt 1 handler 
	jump	__int2_h	; interrupt 2 handler (lowest priority)
	jump	__int0_h	; interrupt 0 handler (highest priority)     
main_init:

;**********************************************************************
; Main program initialization
;**********************************************************************

 

;**********************************************************************
; Initialize the stack pointer
;**********************************************************************
	move	i3h,#HIWORD(_estack)
	move	i3l,#LOWORD(_estack) 

;**********************************************************************
; Initialize the variables in section page0_bss
;**********************************************************************
crt0_init_page0_bss:
	move	i1h, #HIWORD(_spage0bss)
	move	i1l, #LOWORD(_spage0bss)
	move	i2h, #HIWORD(_epage0bss)
	move	i2l, #LOWORD(_epage0bss)
	move	r0,  #BIT0
	jump	crt0_init_page0_bss_cmp 
crt0_init_page0_bss_loop:
	move	(i1)+,r0
crt0_init_page0_bss_cmp:
	cmp     i1h,i2h
	jlt	crt0_init_page0_bss_loop
	cmp	i1l,i2l
	jlt	crt0_init_page0_bss_loop

;**********************************************************************
; Initialize the variables in section  bss
;**********************************************************************
crt0_init_bss:
	move    i1h,#HIWORD(_sbss)
	move	i1l,#LOWORD(_sbss)
	move	i2h,#HIWORD(_ebss)
	move	i2l,#LOWORD(_ebss)
	move	r0, #BIT0
	jump	crt0_init_bss_cmp
crt0_init_bss_loop:
	move	(i1)+,r0
crt0_init_bss_cmp:
	cmp     i1h,i2h
	jlt     crt0_init_bss_loop
	cmp     i1l,i2l
	jlt     crt0_init_bss_loop
	

;**********************************************************************
; End of main program initialization
;**********************************************************************

;**********************************************************************
; Main program
;**********************************************************************
;**********************************************************************
; Enable interrupts and call main
;**********************************************************************
  
	move    stat,#setstat	;enable IRQ0,1,2 & clear evts & ints
  calls	main	  	;call C routine  int main(void)
        
_cleanup:
_exit:
;**********************************************************************
; Enter halt mode
;**********************************************************************  
main_end:
  HALT                	;end program execution
	move    stat,#setstat 	;enable IRQ0,1,2 & clear evts & ints
	jump    main_end    	;just in case..

;**********************************************************************
; End of main program
;**********************************************************************
;**********************************************************************    
; Interrupt handlers
;**********************************************************************
;*****************************************************            
__int1_h:   ; Interrupts level 1
;*****************************************************
        move    -(i3), a           	; backup registers
	sflag
	move    -(i3), a
	move    -(i3), ipl
	move    -(i3), iph
	move    -(i3), i0l
	move    -(i3), i0h
	move    -(i3), i1l
	move    -(i3), i1h
	move    -(i3), i2l
	move    -(i3), i2h
	move    -(i3), r0
	move    -(i3), r1
	move    -(i3), r2
	move    -(i3), r3
  move	r3,RegInt10		; read RegInt10 and clears it (HW magic :-)
	calls	SysInterrupt1Dispatch   ; dispatch interrupt
	clrb    stat, #BIT3     	; clears IN1
  jump	ret_int
  
;**********************************************************************
__int2_h: 	; Interrupts level 2
;**********************************************************************
	move    -(i3), a		; backup registers
	sflag
	move    -(i3), a
	move    -(i3), ipl
	move    -(i3), iph
	move    -(i3), i0l
	move    -(i3), i0h
	move    -(i3), i1l
	move    -(i3), i1h
	move    -(i3), i2l
	move    -(i3), i2h
	move    -(i3), r0
	move    -(i3), r1
	move    -(i3), r2
	move    -(i3), r3
  move	r3,RegInt20		; read RegInt20 and clears it (HW magic :-)
	calls	SysInterrupt2Dispatch   ; dispatch interrupt
	clrb    stat, #BIT4     	; clears IN2
  jump	ret_int
  
;**********************************************************************
__int0_h:	; Interrupts level 0
;**********************************************************************
	move    -(i3), a		; backup registers
	sflag
	move    -(i3), a
	move    -(i3), ipl
	move    -(i3), iph
	move    -(i3), i0l
	move    -(i3), i0h
	move    -(i3), i1l
	move    -(i3), i1h
	move    -(i3), i2l
	move    -(i3), i2h
	move    -(i3), r0
	move    -(i3), r1
	move    -(i3), r2
	move    -(i3), r3
  move	r3,RegInt00		; read RegInt00 and clears it (HW magic :-)
	calls	SysInterrupt0Dispatch   ; dispatch interrupt
	clrb    stat, #BIT2     	; clears IN0
        

;**********************************************************************
ret_int:	
	move    r3, (i3)+		; restores registers
	move    r2, (i3)+
	move    r1, (i3)+
	move    r0, (i3)+
	move    i2h, (i3)+
	move    i2l, (i3)+
	move    i1h, (i3)+
	move    i1l, (i3)+
	move    i0h, (i3)+
	move    i0l, (i3)+
	move    iph, (i3)+
	move    ipl, (i3)+
	rflag   (i3)+
	move    a, (i3)+
	reti		          ; GIE is enabled
        
;**********************************************************************
; End of interrupt handlers
;**********************************************************************        
