// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// App:      EM6812 Soft UART / Monitor
// Version:  1.0 
// Date:     
// Author:   EM Microelectronic
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#include "Periph.H"
#include "Main.H"
#include "Macro.H"



// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Interrupt0, not used 
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void SysInterrupt0Dispatch(unsigned char RegInt) // Priority level 2
{}

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Interrupt1, used for UART RX-Pin / PA7
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void SysInterrupt1Dispatch(unsigned char RegInt)
{
// INT1: Called by crt0.s, var RegInt contains RegInt10 
// Int1_7 = Irq_PA7

  if ((RegInt & Int1_7) > 0)          // Source = PA7 
  {         
     RegOutPB = ~RegOutPB; // debug only !
     RegOutPA       ^= 0x01;
     RegMsk10       &= (0xff-Msk1_7); // Disable Interrupt of RXD pin;       
     vUART_CNT      = cUARTBits;  // 8 bits config
     vUART_STAT     = cUART_RX;   // Reception
     RegTim1Load    = cUART15Bit; // Delay to sample LSB middle
     RegTimersStart |= SWStart1;  // Timer Start
  } 
}

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Interrupt2, used for the Timers
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void SysInterrupt2Dispatch(unsigned char RegInt)
{
// INT2: Called by crt0.s, var RegInt contains RegInt20 
// Int2_0 = Irq_Tim1
  
  RegTim1Load    = cUART1Bit; // Delay to sample next LSB middle
  if ((RegInt & Int2_0) > 0)   // Source = Timer1
  {
      delay; 
      RegOutPB = ~RegOutPB; // debug only !           
      if (  ((vUART_STAT & cUART_TX) == 0)  )  // Transmit or receive?   
        {                                         
          if (vUART_CNT > 0)                       // Receive!
            vUART_BYTE = (vUART_BYTE >> 1) | (RegInPA & cRXPIN_MSK);
          else
            Nop;
        }
      else
        {                                         // Transmit
           if (vUART_CNT == 1)          
             RegOutPA |= cTXPIN_MSK;              // Send Stop bit         
           else if (vUART_CNT > 0)
            {              
              if ((vUART_BYTE & 0x01) > 0) 
                RegOutPA |= cTXPIN_MSK;           // Send 1
              else  
                  RegOutPA &= (0xff-cTXPIN_MSK);  // Send 0
              vUART_BYTE >>= 1;  
            }
        }
      if (vUART_CNT |= 0)
          vUART_CNT--; 
      else
      {
        RegTimersStart &= (0xff - SWStart1);      // Stop Timer
        vUART_STAT |=  cUART_RDY;
        RegMsk10 |= Msk1_7;                       // Reenable PA7 interrupt for next byte
      }    
   }
}

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Return one byte on TXD
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void SendUART(unsigned char Val, unsigned char TXStatus)
{
  while ((vUART_STAT & cUART_RDY) == 0x00) {}; // wait if not ready
  RegMsk10 &= (0xff-Msk1_7);                   // Disable Interrupt of RXD pin  
  vUART_BYTE = Val;
  vUART_STAT = TXStatus;                   // Set Status
  vUART_CNT = cUARTBits+1;                 // 8+1 bits
  RegOutPA &= (0xff-cTXPIN_MSK);           // Set Start bit
  RegTim1Load = cUART1Bit;                 // Load Timer with Bit length 
  RegTimersCfg |= AR1;                     // Autoreload
  RegTimersStart |= SWStart1;              // Restart Timer
}

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// load one byte from RXD
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

unsigned char GetUART(void)
{
  while (vUART_STAT != cUART_RXD) {};      // Wait until SW-UART ready        
  vUART_STAT = cUART_IDLE;
  RegMsk10 &= (0xff-Msk1_7);               // Disable Interrupt of RXD pin  
  return vUART_BYTE;                       // return the received byte
}

// ***************************************************
// Routines for switching the CPU Speed
// ***************************************************
void SetHighSpeed(void)
{
        RegSys1 |= EnRC;                                // Value for 1MHz, RC On
        RegSys2 &= (0xff - Sel32k);                     // Switch to RC
}

void SetLowSpeed(void)
{
        RegSys2 |= Sel32k;                              // switch cpu to 32kHz
        RegSys1 &= (0xff - EnRC);                       // Disable RC   
}

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Execute commands
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void ExecCommand (unsigned char Cmd_name)
{

  if (Cmd_name == cCMD_W_AD)
  {    
    ACCESS_REG.AD_VALUE_HIGH = GetUART();    
    RegMsk10 |= Msk1_7;
    ACCESS_REG.AD_VALUE_LOW  = GetUART();
  }
  else if (Cmd_name == cCMD_W_DATA)
  {
    ACCESS_REG.DATA_VALUE = GetUART();
  }
  else if (Cmd_name == cCMD_R_AD)
  {        
    SendUART(ACCESS_REG.AD_VALUE_HIGH, cUART_TX);
    SendUART(ACCESS_REG.AD_VALUE_LOW, cUART_TX);    
  }
  else if (Cmd_name == cCMD_R_DATA)  
  {
    SendUART(ACCESS_REG.DATA_VALUE, cUART_TX);
  }
  else if ( (Cmd_name == cCMD_UPDATE) | (Cmd_name == cCMD_DUMP) )
  {
    AD_VALUE = ( ((ACCESS_REG.AD_VALUE_HIGH << 8) & 0xFF00) | (ACCESS_REG.AD_VALUE_LOW & 0x00FF) );
    pAD_VALUE = (int*) AD_VALUE;
    if (Cmd_name == cCMD_DUMP)  // dump
      {
        SendUART(*pAD_VALUE, cUART_TX);  
      }
    else 
      {
        *pAD_VALUE =  ACCESS_REG.DATA_VALUE;  // update
      }  
  }

}

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// MAIN, Initialises the EM6812 and handles the 
//       dispatching of the commands received by 
//       the software UART
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
int main(void)
{           


  DisWdog();
  // Init IO & SysRegs
  RegIOSelPB = 0xFF;                       // debug only !
  RegOutPB = 0xFF;                         // debug only !
  RegPullUpPA   = cRXPIN_MSK;              // Enable pulls
  RegPullDownPA ^= cRXPIN_MSK;
  RegIOSelPA   = cTXPIN_MSK | 0x01;        // Only TXPin is output (no floating)
  RegOutPA     = cTXPIN_MSK;               // Only TXPin is '1'
  RegSys1      = EnRC | FreqRange;        // Enable RC
  RegSys2      = EnXtal ;                  // Enable Xtal       
  RegTrimRC    = 0x80;                     // Value for 1MHz
  // Init the vars
  vUART_STAT    = cUART_IDLE; 
  *pAD_VALUE = 0;               
  // Wait for Cold start
  while ((RegSys1 & FlagXtal) == 0) {}        
  // switch Prescaler to 32kHz
  RegPrscCkSel = 0x16;                     // 32K AutoSel mode ... allow to switch off Xtal        
  // Preset the Timers    
  RegTim1Cfg = Clk1Sel_0 | Clk1Sel_2;      // Timer1 Source = 32kHz (Presc1 srce)
  RegTimersCfg = AR1;                      // Autoreload for Timer1        
  // Enable Interrupts
  RegMsk10     = Msk1_7;                   // Enable Interrupts for PA7
  RegMsk20     = Msk2_0;                   // Enable timer interrupt  
  // Start now !                 
  while (1)                                // Main endless loop
    {
      if (vUART_STAT == cUART_RXD)
        {
          vCMD = vUART_BYTE;          
          SendUART(vCMD, cUART_TX);
          if ( (vCMD == cCMD_W_DATA) |     // Check for valid command 
               (vCMD == cCMD_R_DATA) | 
               (vCMD == cCMD_W_AD)   | 
               (vCMD == cCMD_R_AD)   |
               (vCMD == cCMD_UPDATE) |
               (vCMD == cCMD_DUMP)   )                
                  ExecCommand(vCMD);                                         
          while ((vUART_STAT & cUART_RDY)== 0x00) {}; // Wait until UART terminated 
          // Enable Interrupts
          RegMsk10     = Msk1_7;                      // Re-Enable Interrupts for PA7 
        }      
        Halt;                                         // Stop the cpu and wait for interrupts
      }
}



   
